#!./uwsgi --import decoratortest -s :3035 -M --spooler myspool --enable-threads -p 8 --spooler-ordered
import uwsgi

from uwsgidecorators import *

from uwsgicc import app as application

import time


# register rpc function helloworld
@rpc("helloworld")
def hello2():
    return "[RPC] Hello World"

# register signal 1
@signal(1)
def what_time_is_it(num):
    print(time.asctime())

# register signal 100
@signal(100, target='workers')
def what_time_is_it(num):
    print("*** I AM THE WORKER %d AT %s ***" % (uwsgi.worker_id(), time.asctime()))


# a 3 seconds timer
@timer(3)
def salut(num):
    print("Salut !!! 3 seconds elapsed and signal %d raised" % num)

# a 10 seconds red black timer (executed by the spooler)
@rbtimer(10, target='spooler')
def tenseconds(num):
    print("red black timer !!! 10 seconds elapsed and signal %d raised" % num)

# monitor /tmp directory
@filemon("/tmp")
def tmpmodified(num):
    print("/tmp has been modified")


# spool a long running task
@spool
def a_long_task(args):
    for i in xrange(1,10):
        print("%s = %d" % ( str(args), i))
        print(uwsgi.call('helloworld'))
        time.sleep(1)

# continuosly spool a long running task
@spoolforever
def an_infinite_task(args):
    for i in xrange(1,4):
        print("infinite: %d %s" % (i, str(args)))
        print(uwsgi.call('helloworld'))
        uwsgi.signal(100)
        time.sleep(1)


# spool a task after 5 seconds
@spool
def delayed_task(args):
    print("*** I am a delayed spool job. It is %s [%s]***" % (time.asctime(), str(args)))
    # send a signal to all workers
    uwsgi.signal(100)

@spool
def big_body_task(args):
    print("*** managing a task with a body of %d bytes ***" % len(args['body']))
    print(args['body'].swapcase()) 

# run a task every hour
@cron(59, -1, -1, -1, -1)
def one_hour_passed(num):
    print("received signal %d after 1 hour" % num)

@thread
def a_running_thread():
    while True:
        time.sleep(2)
        print("i am a no-args thread")

@thread
def a_running_thread_with_args(who):
    while True:
        time.sleep(2)
        print("Hello %s (from arged-thread)" % who)

@postfork
@thread
def a_post_fork_thread():
    while True:
        time.sleep(3)
        if uwsgi.i_am_the_spooler():
            print("Hello from a thread in the spooler")
        else:
            print("Hello from a thread in worker %d" % uwsgi.worker_id())

@postfork
def fork_happened():
    print("fork() has been called [1]")

@postfork
def fork_happened2():
    if uwsgi.i_am_the_spooler():
        return
    print("worker %d is waiting for signal 100..." % uwsgi.worker_id())
    uwsgi.signal_wait(100)
    print("worker %d received signal %d" % (uwsgi.worker_id(), uwsgi.signal_received()))
    print("fork() has been called [2] wid: %d" % uwsgi.worker_id())

@postfork
@lock
def locked_func():
    print("starting locked function on worker %d" % uwsgi.worker_id())
    for i in xrange(1, 5):
        time.sleep(1)
        print("[locked %d] waiting..." % uwsgi.worker_id())
    print("done with locked function on worker %d" % uwsgi.worker_id())

print(uwsgi.call('helloworld'))
spool_filename = a_long_task.spool({'foo':'bar'}, hello='world')
print("spool filename = %s" % spool_filename)
an_infinite_task.spool(foo='bar', priority=3)
when = int(time.time())+5
print("scheduling a delayed task at %d" % when)
delayed_task.spool(foo2='bar2', at=when)
a_running_thread()
a_running_thread_with_args("uWSGI")
uwsgi_source_file = open('uwsgi.c','r')
print(big_body_task.spool(priority=9,filename='uwsgi.c',body=uwsgi_source_file.read()))
uwsgi_source_file.close()



import sys
import os
import pyuwsgi

orig_args = sys.argv
orig_executable = sys.executable
orig_args.insert(0, orig_executable)

uwsgi_args = []

for arg in sys.argv[2:]:
    uwsgi_args.append(arg)

#pyuwsgi.run('welcome.ini')
pyuwsgi.run(uwsgi_args)

# if you are here uWSGI has been reloaded
os.execv(orig_executable, orig_args)



import uwsgi

from mako.template import Template
import time


def application(env, start_response):
	start_response( '200 OK', [ ('Content-Type','text/html') ])

	mytemplate = Template("<h1>I am Mako at ${thetime}</h1>")
	uwsgi.green_schedule()

	yield mytemplate.render(thetime = time.time() )

	for i in range(1, 100):
		mytemplate = Template("Iteration ${thei} at ${thetime}<br/>")
		uwsgi.green_schedule()
		yield mytemplate.render(thei = i, thetime = time.time() )
	



import uwsgi

import time
import sys
import os


def application(env, start_response):
    print env
    start_response('200 OK', [('Content-Type', 'text/html')])

    yield '<h1>uWSGI %s status</h1>' % uwsgi.version
    yield 'masterpid: <b>' + str(uwsgi.masterpid()) + '</b><br/>'

    yield 'started on: <b>' + time.ctime(uwsgi.started_on) + '</b><br/>'

    yield 'buffer size: <b>' + str(uwsgi.buffer_size) + '</b><br/>'

    yield 'total_requests: <b>' + str(uwsgi.total_requests()) + '</b><br/>'

    yield 'log size: <b>' + str(uwsgi.logsize()) + '</b><br/>'

    yield 'workers: <b>' + str(uwsgi.numproc) + '</b><br/>'

    yield "cwd: <b>%s</b><br/>" % os.getcwd()

    try:
        yield "mode: <b>%s</b><br/>" % uwsgi.mode
    except:
        pass

    try:
        yield "pidfile: <b>%s</b><br/>" % uwsgi.pidfile
    except:
        pass

    yield "<h2>Hooks</h2>"

    for h in range(0,255):
        if uwsgi.has_hook(h):
            yield "%d<br/>" % h

    yield '<h2>dynamic options</h2>'

    yield '<table border="1">'
    yield '<th>worker id</th><th>pid</th><th>in request</th><th>requests</th><th>running time</th><th>address space</th><th>rss</th>'

    workers = uwsgi.workers();

    yield '<h2>workers</h2>'

    for w in workers:
        #print w
        #print w['running_time']
        if w is not None:
            yield '<tr><td>'+ str(w['id']) +'</td><td>' + str(w['pid']) + '</td><td>' + str(w['pid']) + '</td><td>' + str(w['requests']) + '</td><td>' + str(w['running_time']) + '</td><td>' + str(w['vsz']) + '</td><td>' + str(w['rss']) + '</td></tr>'
            print w

    yield '</table>'

    yield "<h2>PYTHONPATH</h2>"

    yield "<ul>"
    for p in sys.path:
        yield "<li>%s</li>" % p

    yield "</ul>"

    yield "<i>%s</i>" % str(os.uname())




import uwsgi

current_node = 0

def application(e,s):

	global current_node

	nodes = uwsgi.cluster_nodes()
	print nodes

	if len(nodes) == 0:
		print "no cluster node available"
		raise StopIteration

	if current_node >= len(nodes):
		current_node = 0

	node = nodes[current_node]

	for part in uwsgi.send_message(node, 0, 0, e, 0, e['wsgi.input'].fileno(), uwsgi.cl()):
		yield part 

	current_node+=1



NAME='pty'

import os
uwsgi_os = os.uname()[0]

CFLAGS = []
LDFLAGS = []
if uwsgi_os in ('Linux', 'FreeBSD', 'GNU', 'NetBSD', 'DragonFly'):
    LIBS = ['-lutil']
else:
    LIBS = []
GCC_LIST = ['pty']



# encoding: utf-8

"""
This is a hack allowing you installing
uWSGI and uwsgidecorators via pip and easy_install
since 1.9.11 it automatically detects pypy
"""

import os
import sys
import errno
import shlex
import uwsgiconfig

from setuptools import setup, Extension
from setuptools.command.build_ext import build_ext


class uWSGIBuildExt(build_ext):

    UWSGI_NAME = 'uwsgi'
    UWSGI_PLUGIN = 'pyuwsgi'

    def build_extensions(self):
        self.uwsgi_setup()
        # XXX: needs uwsgiconfig fix
        self.uwsgi_build()
        if 'UWSGI_USE_DISTUTILS' not in os.environ:
            # XXX: needs uwsgiconfig fix
            # uwsgiconfig.build_uwsgi(self.uwsgi_config)
            return

        else:
            # XXX: needs uwsgiconfig fix
            os.unlink(self.uwsgi_config.get('bin_name'))

        # FIXME: else build fails :(
        for baddie in set(self.compiler.compiler_so) & set(('-Wstrict-prototypes',)):
            self.compiler.compiler_so.remove(baddie)

        build_ext.build_extensions(self)

    def uwsgi_setup(self):
        default = 'pypy' if '__pypy__' in sys.builtin_module_names else 'default'
        profile = os.environ.get('UWSGI_PROFILE') or 'buildconf/%s.ini' % default

        if not profile.endswith('.ini'):
            profile = profile + '.ini'
        if '/' not in profile:
            profile = 'buildconf/' + profile

        # FIXME: update uwsgiconfig to properly set _EVERYTHING_!
        config = uwsgiconfig.uConf(profile)
        # insert in the beginning so UWSGI_PYTHON_NOLIB is exported
        # before the python plugin compiles
        ep = config.get('embedded_plugins').split(',')
        if self.UWSGI_PLUGIN in ep:
            ep.remove(self.UWSGI_PLUGIN)
        ep.insert(0, self.UWSGI_PLUGIN)
        config.set('embedded_plugins', ','.join(ep))
        config.set('as_shared_library', 'true')
        config.set('bin_name', self.get_ext_fullpath(self.UWSGI_NAME))
        try:
            os.makedirs(os.path.dirname(config.get('bin_name')))
        except OSError as e:
            if e.errno != errno.EEXIST:
                raise

        self.uwsgi_profile = profile
        self.uwsgi_config = config

    def uwsgi_build(self):
        uwsgiconfig.build_uwsgi(self.uwsgi_config)

        # XXX: merge uwsgi_setup (see other comments)
        for ext in self.extensions:
            if ext.name == self.UWSGI_NAME:
                ext.sources = [s + '.c' for s in self.uwsgi_config.gcc_list]
                ext.library_dirs = self.uwsgi_config.include_path[:]
                ext.libraries = list()
                ext.extra_compile_args = list()

                for x in uwsgiconfig.uniq_warnings(
                    self.uwsgi_config.ldflags + self.uwsgi_config.libs,
                ):
                    for y in shlex.split(x):
                        if y.startswith('-l'):
                            ext.libraries.append(y[2:])
                        elif y.startswith('-L'):
                            ext.library_dirs.append(y[2:])

                for x in self.uwsgi_config.cflags:
                    for y in shlex.split(x):
                        if y:
                            ext.extra_compile_args.append(y)


setup(
    name='uWSGI',
    license='GPL2',
    version=uwsgiconfig.uwsgi_version,
    author='Unbit',
    author_email='info@unbit.it',
    description='The uWSGI server',
    cmdclass={
        'build_ext': uWSGIBuildExt,
        },
    py_modules=[
        'uwsgidecorators',
        ],
    ext_modules=[
        Extension(uWSGIBuildExt.UWSGI_NAME, sources=[]),
        ],
    entry_points={
        'console_scripts': ['uwsgi=%s:run' % uWSGIBuildExt.UWSGI_NAME],
        },
    )



import uwsgi

def application(env, start_response):

	# open the socket
	fd = uwsgi.async_connect("192.168.173.100:3032")

	# wait for connection ready (3s timeout)
	yield uwsgi.wait_fd_write(fd, 3)

	# has timed out ?
	if env['x-wsgiorg.fdevent.timeout']:
		print "connection timed out !!!"
		uwsgi.close(fd)
		raise StopIteration

	# connection refused ?
	if not uwsgi.is_connected(fd):
		print "unable to connect"
		uwsgi.close(fd)
		raise StopIteration
	

	# send request
	# env can contains python objects, but send_message will discard them.
	# In this way we will automagically have a congruent and valid uwsgi packet
	uwsgi.async_send_message(fd, 0, 0, env)

	# send the http body
	# ready body in async mode and resend to fd	
	# uwsgi.recv will use always an internal buffer of 4096, but can be limited in the number of bytes to read

	# does this request has a body ?
	cl = uwsgi.cl()

	if cl > 0:
		# get the input fd
		input = env['wsgi.input'].fileno()

		# read (in async mode) upto 'cl' data and send to uwsgi peer
		while cl > 0:
			bufsize = min(cl, 4096)
			yield uwsgi.wait_fd_read(input, 30)
			if env['x-wsgiorg.fdevent.timeout']:
				print "connection timed out !!!"
				uwsgi.close(fd)
				raise StopIteration
			body = uwsgi.recv(input, bufsize)
			if body:
				uwsgi.send(fd, body)
				cl = cl - len(body)
			else:
				break
	

	# wait for response (30s timeout)
	yield uwsgi.wait_fd_read(fd, 30)

	# has timed out ?
	if env['x-wsgiorg.fdevent.timeout']:
		print "connection timed out !!!"
		uwsgi.close(fd)
		raise StopIteration

	data = uwsgi.recv(fd)
	# recv the data, if it returns None the callable will end
	while data:
		yield data
		# wait for response
		yield uwsgi.wait_fd_read(fd, 30)
		if env['x-wsgiorg.fdevent.timeout']:
			print "connection timed out !!!"
			uwsgi.close(fd)
			raise StopIteration
		data = uwsgi.recv(fd)

	uwsgi.close(fd)
	



# https://github.com/unbit/uwsgi/pull/2615
# atexit should be called when reached max-requests.
#
# Start this app:
#
#   $ ./uwsgi --http-socket :8000 --master -L --wsgi-file=tests/threads_atexit.py \
#       --workers 1 --threads 32 --max-requests 40 --min-worker-lifetime 6 --lazy-apps
#
# Access to this app with hey[1]:
#
#   # Do http access for 5 minutes with 32 concurrency
#   $ ./hey -c 32 -z 5m 'http://127.0.0.1:8000/'
#
# Search how many stamp files:
#
#   $ ls uwsgi_worker*.txt | wc -l
#   39  # should be 0
#
# [1] https://github.com/rakyll/hey

import atexit
import os
import sys
import time


pid = os.getpid()
stamp_file = f"./uwsgi_worker{pid}.txt"


with open(stamp_file, "w") as f:
    print(time.time(), file=f)


@atexit.register
def on_finish_worker():
    print(f"removing {stamp_file}", file=sys.stderr)
    os.remove(stamp_file)


def application(env, start_response):
    time.sleep(1)
    start_response('200 OK', [('Content-Type', 'text/html')])
    return [b"Hello World"]



import threading
import time


def run():
    print("[DEADLOCKS] started run")
    st = time.time()
    while time.time() < st + 5:
        pass
    print("[DEADLOCKS] finished run")

t = threading.Thread(target=run)
t.daemon = True
t.start()



import uwsgi

uwsgi.log("I am uWSGI %s" % uwsgi.version)

def application(env, start_response):
	start_response('200 OK', [('Content-Type','text/html')])
	uwsgi.log(str(env))

	if env['PATH_INFO'] == '/logme':
		uwsgi.log_this_request()
	return "log written"



import uwsgi
import time


def application(env, start_response):
    sleepvalue = 5
    if env.has_key('QUERY_STRING'):
        if env['QUERY_STRING'] != '':
            sleepvalue = int(env['QUERY_STRING'])
        start_response('200 Ok', [('Content-type', 'text/html')])
    start_at = time.time()
    uwsgi.green_sleep(sleepvalue)
    #print "TIMEOUT: ", env['x-wsgiorg.fdevent.timeout']
    yield "<h1>Hello World after %s seconds</h1>" % str(time.time() - start_at)



import os
import shutil
import subprocess

NAME='jvm'

JVM_INCPATH = None
JVM_LIBPATH = None

operating_system = os.uname()[0].lower()

try:
    arch = os.environ['JVM_ARCH']
except:
    arch = os.uname()[4].lower()

if arch in ('i686', 'x86', 'x86_32'):
    arch = 'i386'
elif arch in ('x86_64',):
    arch = 'amd64'
elif arch.startswith('arm'):
    arch = 'arm'

# try to detect the JVM
if operating_system == 'darwin':
    known_jvms = ('/System/Library/Frameworks/JavaVM.framework/Headers',)
    for jvm in known_jvms:
        if os.path.exists(jvm):
            JVM_INCPATH = ["-Wno-deprecated-declarations", "-I%s" % jvm]
            JVM_LIBPATH = ["-framework JavaVM"]
elif operating_system.startswith('cygwin'):
    JVM_INCPATH = ['-I"/cygdrive/c/Program Files/Java/jdk1.7.0_17/include"', '-I"/cygdrive/c/Program Files/Java/jdk1.7.0_17/include/win32"']
    JVM_LIBPATH = ['-L"/cygdrive/c/Program Files/Java/jdk1.7.0_17/jre/bin/server"']
else:
    known_jvms = ('/usr/lib/jvm/java-7-openjdk', '/usr/local/openjdk7', '/usr/lib/jvm/java-6-openjdk', '/usr/local/openjdk', '/usr/java', '/usr/lib/jvm/java/', '/usr/lib/jvm/java-8-openjdk-%s' % arch, '/usr/lib/jvm/java-11-openjdk-%s' % arch)
    for jvm in known_jvms:
        if os.path.exists(jvm + '/include'):
            JVM_INCPATH = ["-I%s/include/" % jvm, "-I%s/include/%s" % (jvm, operating_system)]
            if os.path.exists("%s/jre" % jvm):
                JVM_LIBPATH = ["-L%s/jre/lib/%s/server" % (jvm, arch)]
            else:
                JVM_LIBPATH = ["-L%s/lib/server" % (jvm,)]
            break
        if os.path.exists("%s-%s/include" % (jvm, arch)):
            jvm = "%s-%s" % (jvm, arch)
            JVM_INCPATH = ["-I%s/include/" % jvm, "-I%s/include/%s" % (jvm, operating_system)]
            JVM_LIBPATH = ["-L%s/jre/lib/%s/server" % (jvm, arch)]
            break

try:
    JVM_INCPATH = ['-I"' + os.environ['UWSGICONFIG_JVM_INCPATH'] + '"']
except:
    pass

try:
    JVM_LIBPATH = ['-L"' + os.environ['UWSGICONFIG_JVM_LIBPATH'] + '"']
except:
    pass

if not JVM_INCPATH or not JVM_LIBPATH:
    print("unable to autodetect the JVM path, please specify UWSGICONFIG_JVM_INCPATH and UWSGICONFIG_JVM_LIBPATH environment vars")
    os._exit(1)

CFLAGS = JVM_INCPATH
LDFLAGS = JVM_LIBPATH
LIBS = ['-ljvm']
if "-framework JavaVM" in JVM_LIBPATH:
    LIBS = []

GCC_LIST = ['jvm_plugin']

if 'LD_RUN_PATH' in os.environ:
    os.environ['LD_RUN_PATH'] += ':' + JVM_LIBPATH[0][2:]
else:
    os.environ['LD_RUN_PATH'] = JVM_LIBPATH[0][2:]

def post_build(config):
    if subprocess.call("javac %s/plugins/jvm/uwsgi.java" % os.getcwd(), shell=True) != 0:
        os._exit(1)
    if subprocess.call("cd %s/plugins/jvm ; jar cvf uwsgi.jar *.class" % os.getcwd(), shell=True) != 0:
        os._exit(1)
    print("*** uwsgi.jar available in %s/plugins/jvm/uwsgi.jar ***" % os.getcwd())

    env = os.environ.get('VIRTUAL_ENV')
    if env:
        src = "%s/plugins/jvm/uwsgi.jar" % os.getcwd()
        tgt = "%s/lib/uwsgi.jar" % env
        shutil.copyfile(src, tgt)
        print("*** uwsgi.jar had been copied to %s" % tgt)

        plugin = "%s/jvm_plugin.so" % os.getcwd()
        if os.path.exists(plugin):
            tgt = "%s/bin/jvm_plugin.so" % env
            shutil.copyfile(plugin, tgt)
            print("*** jvm_plugin.so had been copied to %s" % tgt)

